/************************************************************************
*                                                                       *
*         Simple Desktop Display Manager Theme for UNSW Sydney          *
*                 Copyright (C) 2016-19, John Zaitseff                  *
*                                                                       *
************************************************************************/

/*
  Author: John Zaitseff <J.Zaitseff@zap.org.au>
  $Id: 21adaf0c4f48c75955c0964135b0dc0cb7b8b913 $

  The Simple Desktop Display Manager Theme for UNSW Sydney provides a
  theme for SDDM that is suitable for the University of New South Wales,
  Australia.


  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see https://www.gnu.org/licenses/.
*/


import QtQuick 2.0
import SddmComponents 2.0


Rectangle {
    id: mainContainer
    width: 1024
    height: 768

    property string defaultFontFamily: "Oxygen"

    LayoutMirroring.enabled: Qt.application.layoutDirection === Qt.RightToLeft
    LayoutMirroring.childrenInherit: true

    TextConstants {
	id: textConstants
    }

    Connections {
	target: sddm

	onLoginSucceeded: {
	    errorMessage.color = "lawngreen"
	    errorMessage.font.bold = false
	    errorMessage.text = textConstants.loginSucceeded
	}

	onLoginFailed: {
	    errorMessage.color = "red"
	    errorMessage.font.bold = true
	    errorMessage.text = textConstants.loginFailed
	}
    }

    Background {
	anchors.fill: parent
	source: config.background
	fillMode: Image.PreserveAspectCrop

	onStatusChanged: {
	    if (status == Image.Error && source != config.defaultBackground) {
		source = config.defaultBackground
	    }
	}
    }

    Rectangle {
	id: mainWindow
	anchors.fill: parent
	color: "transparent"
	visible: primaryScreen

	Column {
	    id: clock
	    anchors.margins: 10
	    anchors.top: parent.top
	    anchors.right: parent.right

	    property date dateTime: new Date()

	    Timer {
		running: true
		repeat: true
		interval: 250
		onTriggered: clock.dateTime = new Date()
	    }

	    Text {
		id: time
		text: Qt.formatTime(clock.dateTime)
		anchors.horizontalCenter: parent.horizontalCenter
		font.family: defaultFontFamily
		font.pointSize: 72
		color: "#80999999"
	    }

	    Text {
		id: date
		text: Qt.formatDate(clock.dateTime, Qt.DefaultLocaleLongDate)
		anchors.horizontalCenter: parent.horizontalCenter
		font.family: defaultFontFamily
		font.pointSize: 24
		color: "#80999999"
	    }
	}

	Image {
	    id: loginBox
	    anchors.centerIn: parent
	    width: Math.max(320, mainColumn.implicitWidth + 50)
	    height: Math.max(320, mainColumn.implicitHeight + 50)

	    source: "rectangle.png"

	    Column {
		id: mainColumn
		anchors.centerIn: parent
		spacing: 12

		Text {
		    id: hostNameLabel
		    anchors.horizontalCenter: parent.horizontalCenter
		    width: parent.width
		    height: text.implicitHeight
		    // text: textConstants.welcomeText.arg(sddm.hostName)
		    // text: "L-UNSW00012345"
		    text: sddm.hostName
		    font.family: defaultFontFamily
		    font.pixelSize: 28
		    font.bold: true
		    color: "black"
		    verticalAlignment: Text.AlignVCenter
		    horizontalAlignment: Text.AlignHCenter
		}

		Column {
		    width: parent.width
		    spacing: 4

		    Text {
			id: nameLabel
			width: parent.width
			text: textConstants.userName
			font.family: defaultFontFamily
			font.bold: true
			font.pixelSize: 12
		    }

		    TextBox {
			id: name
			width: parent.width
			height: 30
			// text: userModel.lastUser
			font.family: defaultFontFamily
			font.pixelSize: 14

			KeyNavigation.backtab: rebootButton
			KeyNavigation.tab: password

			Keys.onPressed: {
			    if (event.key === Qt.Key_Return || event.key === Qt.Key_Enter) {
				sddm.login(name.text, password.text, session.index)
				event.accepted = true
			    }
			}
		    }
		}

		Column {
		    width: parent.width
		    spacing: 4

		    Text {
			id: passwordLabel
			width: parent.width
			text: textConstants.password
			font.family: defaultFontFamily
			font.bold: true
			font.pixelSize: 12
		    }

		    PasswordBox {
			id: password
			width: parent.width
			height: 30
			font.family: defaultFontFamily
			font.pixelSize: 14

			KeyNavigation.backtab: name
			KeyNavigation.tab: session

			Keys.onPressed: {
			    if (event.key === Qt.Key_Return || event.key === Qt.Key_Enter) {
				sddm.login(name.text, password.text, session.index)
				event.accepted = true
			    }
			}
		    }
		}

		Row {
		    width: parent.width / 2
		    spacing: 8
		    z: 100

		    Column {
			width: parent.width * 1.25
			spacing: 4
			z: 100
			anchors.bottom: parent.bottom

			Text {
			    id: sessionLabel
			    width: parent.width
			    text: textConstants.session
			    wrapMode: TextEdit.WordWrap
			    font.family: defaultFontFamily
			    font.bold: true
			    font.pixelSize: 12
			}

			ComboBox {
			    id: session
			    width: parent.width
			    height: 30
			    font.family: defaultFontFamily
			    font.pixelSize: 14
			    arrowIcon: "down.png"

			    model: sessionModel
			    index: sessionModel.lastIndex

			    KeyNavigation.backtab: password
			    KeyNavigation.tab: layoutBox
			}
		    }

		    Column {
			width: parent.width * 0.7
			spacing: 4
			z: 101
			anchors.bottom: parent.bottom

			Text {
			    id: layoutLabel
			    width: parent.width
			    text: textConstants.layout
			    wrapMode: TextEdit.WordWrap
			    font.family: defaultFontFamily
			    font.bold: true
			    font.pixelSize: 12
			}

			LayoutBox {
			    id: layoutBox
			    width: parent.width
			    height: 30
			    font.family: defaultFontFamily
			    font.pixelSize: 14
			    arrowIcon: "down.png"

			    KeyNavigation.backtab: session
			    KeyNavigation.tab: loginButton
			}
		    }
		}

		Column {
		    width: parent.width

		    Text {
			id: errorMessage
			anchors.horizontalCenter: parent.horizontalCenter
			text: textConstants.prompt
			font.family: defaultFontFamily
			font.pixelSize: 12
			font.bold: false
			color: "white"
		    }
		}

		Row {
		    spacing: 8
		    anchors.horizontalCenter: parent.horizontalCenter

		    property int buttonWidth: Math.max(loginButton.implicitWidth,
						       shutdownButton.implicitWidth,
						       rebootButton.implicitWidth,
						       80) + 8

		    Button {
			id: loginButton
			width: parent.buttonWidth
			text: textConstants.login
			font.family: defaultFontFamily
			font.bold: false

			onClicked: {
			    sddm.login(name.text, password.text, session.index)
			}

			KeyNavigation.backtab: layoutBox
			KeyNavigation.tab: shutdownButton
		    }

		    Button {
			id: shutdownButton
			width: parent.buttonWidth
			text: textConstants.shutdown
			font.family: defaultFontFamily
			font.bold: false

			onClicked: {
			    sddm.powerOff()
			}

			KeyNavigation.backtab: loginButton
			KeyNavigation.tab: rebootButton
		    }

		    Button {
			id: rebootButton
			width: parent.buttonWidth
			text: textConstants.reboot
			font.family: defaultFontFamily
			font.bold: false

			onClicked: {
			    sddm.reboot()
			}

			KeyNavigation.backtab: shutdownButton
			KeyNavigation.tab: name
		    }
		}
	    }
	}
    }

    Component.onCompleted: {
	mainWindow.forceActiveFocus()
	if (name.text == "") {
	    name.focus = true
	} else {
	    password.focus = true
	}
    }

    Timer {
	running: true
	repeat: false
	interval: 250
	onTriggered: {
	    mainWindow.forceActiveFocus()
	    if (name.text == "") {
		name.focus = true
	    } else {
		password.focus = true
	    }
	}
    }
}
